<?php
define('SI_RECAPTCHA_ADDON_DIR', str_replace('\\', '/',dirname(__FILE__)));
define('SI_RECAPTCHA_ADDON_URL', str_replace(SI_PLUGIN_DIR, SI_PLUGIN_URL, SI_RECAPTCHA_ADDON_DIR ));

class siRecaptchaAddon extends SourceImmoAddon{
    public $_addon_path = SI_RECAPTCHA_ADDON_DIR;

    public function __construct(){
        $this->name = 'recaptcha';
        $this->caption = 'reCaptcha';
        $this->default_configuration = array(
            // Used method to display listings' details
            'site_key' => '', // Google reCaptcha site key
            // 'secret' => '', 
            // Loading text while waiting for server to responds
            // Timeout before switching back to local details (in sec)
        );
    }

    public function register_hooks(){
        if (!is_admin() ){
            // Add a hook for listing 
            
            add_action('si/contact-form/end', [$this, 'generate_recaptcha']);

            add_action('wp_enqueue_scripts', [$this, 'register_resources']);

            add_filter('si/contact-form/validate', [$this, 'validate_recaptcha'], 10, 2);
        }
    }

    public function register_resources(){
        
        //wp_enqueue_script( 'si-recaptcha-addon-script', plugins_url('/addons/recaptcha/assets/recaptcha.min.js', SI_PLUGIN), null, filemtime(SI_PLUGIN_DIR . '/addons/recaptcha/assets/recaptcha.min.js') );
        //wp_enqueue_style( 'si-recaptcha-addon-style', plugins_url('/addons/recaptcha/assets/recaptcha.min.css', SI_PLUGIN), null, filemtime(SI_PLUGIN_DIR . '/addons/recaptcha/assets/recaptcha.min.css') );
        $style_version = filemtime(SI_RECAPTCHA_ADDON_DIR . "/assets/style.min.css");
        $style_url = SI_RECAPTCHA_ADDON_URL . "/assets/style.min.css";
        wp_enqueue_style( 'si-addon-recaptcha-style', $style_url, null, $style_version);

        $script_version = filemtime(SI_RECAPTCHA_ADDON_DIR . "/assets/script.js");
        $script_url = SI_RECAPTCHA_ADDON_URL . "/assets/script.js";
        wp_enqueue_script( 'si-addon-recaptcha-script', $script_url, ['si-public-app'], $script_version,true);

        wp_enqueue_script( 'google-recaptcha', 'https://www.google.com/recaptcha/api.js?onload=SourceImmoRecaptchaModuleInit&render=explicit', array('si-addon-recaptcha-script'), null, ['async' => true, 'defer' => true]);
    }


    public function generate_recaptcha(){
        include SI_RECAPTCHA_ADDON_DIR . '/views/recaptcha_element.php';
    }

    public function validate_recaptcha($isValid, $formData=null, $formMatedata=null){
        $recaptcha = isset($_POST['g-recaptcha-response']) ? $_POST['g-recaptcha-response'] :  $formData->recaptcha_response;
        $secret = get_option('si/recaptcha/secret');
        if (empty($recaptcha) || empty($secret) ) {
            error_log('reCaptcha validation failed');
            return false;
        }

        $response = file_get_contents("https://www.google.com/recaptcha/api/siteverify?secret=$secret&response=$recaptcha");
        $responseKeys = json_decode($response,true);
        if(intval($responseKeys["success"]) !== 1) {
            error_log('reCaptcha validation failed');
            $isValid = false;
        }

        return $isValid;
    }

}

SourceImmo::current()->addons->register(new siRecaptchaAddon());