<?php
class SourceImmoPageBuilder{
    public $layout = '';
    public $rendered_once = false;
    public $type = '';
    public $page_template_rendered = false;
    public $locale = 'en';
    public $inline_content = '';

    public function __construct($type){
        $this->type = $type;
        $this->layout = SourceImmo::current()->get_detail_layout($type);
        $this->locale = substr(get_locale(),0,2);
        
        //echo('layout:' . );

        remove_filter( 'the_content', 'wpautop' );
        remove_filter( 'the_content', 'wptexturize');
        
        // add_action('si_start_of_template', array($this, 'start_of_template'), 10, 1);
        // add_action('si_end_of_template', array($this, 'end_of_template'), 10, 0);

        //add_filter( 'the_content', array($this, 'get_page_content'), 0);
        add_filter( 'body_class', function($classes){
            if(isset($this->layout->isolation)){
                $layoutIsolation = strtolower($this->layout->isolation);
                $classes[] = "si-layout-isolation-{$layoutIsolation}";
            }
           
        
            return $classes; 
        });
        
        
    }

    public function start_page(){
        
        //ob_start();
    }

    public function close_page(){
        $this->inline_content = ob_get_contents();
        //ob_clean();

        
    }

    public function getLayoutPage($page_id){
        return self::findPage($page_id, $this->type, $this->locale);
    }

    public function get_page_content($content){
        if($this->rendered_once) return;
        //__c($content);
        
        $result = $content . $this->inline_content;

        $this->rendered_once = true;
        $result = do_shortcode($result);

        return $result;
    }

    public function get_page_template($page_id){
        global $wp_query;
        

        //__c($wp_query->is_home);

        $page_template = '';
        if($page_id !== null){
            if(is_numeric($page_id)){
                $page_template = get_page_template_slug($page_id);
            }
        }
        $page_template = apply_filters('si/page_builder/get_page_template',$page_template);
        
        
        if($page_template != '' && $page_template !== false){ 
            //echo($page_template);
            return $page_template;
        }

        //$templateFiles = wp_get_theme()->get_files('php',0,true);
        $page_template_list = ['page.php','single.php','index.php'];
        $path = locate_template($page_template_list);
        if($path != '') return $path;
        // $priorityList = array('page','single','index');

        // foreach ($priorityList as $pageName) {
        //     foreach($templateFiles as $template => $path){
        //         if($template== $pageName . '.php'){
        //             return $path;
        //         }
        //     }
        // }

        return null;
    }

    public function getSingleControllerName(){
        $normalizeType = ucfirst($this->type);

        return "single{$normalizeType}Ctrl";
    }
    public function getSingleClassName(){
        $normalizeType = strtolower($this->type);
        return "{$normalizeType}-single";
    }

    public function render(){
        $page_id = isset($this->layout->page) ? $this->layout->page : null;
        global $post; 

        $post = $this->getLayoutPage($page_id);
        if($post !== null) $page_id = $post->ID;

        do_action('si_page_builder_prerender', $page_id);
        do_action('si/page_builder/prerender', $page_id);

        $pageTemplate = $this->get_page_template($page_id);
       
        $isolation = SourceImmo::current()->get_isolation($this->type);
        if($isolation != 'ISOLATE'){
            add_filter( 'body_class',function($classes){
                $className = $this->getSingleClassName();
                $classes[] = "{{model!=null?'si-loaded':''}} si-single-content {$className}";
                return $classes;
            }, 1 );
            
            add_filter( 'si/page-builder/body-attributes', function($attrs){
                $ref_number = get_query_var( 'ref_number');
                $controllerName = $this->getSingleControllerName();

                $attrs["data-ng-controller"] = $controllerName;
                $attrs["data-ng-init"] = "init('{$ref_number}')";
                $attrs["ng-cloak"] = '';

                return $attrs;
            });

            $body_start_actions = apply_filters('si/page-builder/body-start',[]);
            
            if(!did_action('si/page-builder/body-start-begin')){
                $body_start_actions[] = 'wp_body_open';
            }

            if(count($body_start_actions) > 0){
                foreach ($body_start_actions as $action) {
                    add_action( $action, function() {
                        $ref_number = get_query_var( 'ref_number');
                        $controllerName = $this->getSingleControllerName();
                        $className = $this->getSingleClassName();
                        $controller = "data-ng-controller=\"{$controllerName}\"  data-ng-init=\"init('{$ref_number}')\"";

                        if(did_filter('si/page-builder/body-attributes') >=1 ){
                            $controller = '';
                        }

                        $containerStart = "<div ng-cloak {$controller} class=\"{{model!=null?'si-loaded':''}} si-single-content {$className}\"><div class=\"si-content\">";
                        echo($containerStart);
                    } );
                }
            }

            $body_end_actions = apply_filters('si/page-builder/body-end',[]);
            if(count($body_start_actions)>0 && count($body_end_actions) == 0){
                $body_end_actions[] = 'wp_footer';
            }
            
            foreach ($body_end_actions as $action) {
                add_action( $action, function() {
                    echo("</div></div>");
                } );
            }
        }

        if($pageTemplate == null){
            wp_head();
        }
        
        
        if(($post != null && $post->post_content == '') || ($post != null && $post->post_type != 'page')){
            
            $post->post_content = "[si_{$this->type}]";
            
        }
        

        
        $this->setupPost($post);
        

        add_filter('the_title', function($title,$id=null) use($page_id){
            global $post;
            if( ($id!=null && $id == $page_id) ){
                return ''; //apply_filters('si_page_title', $title);
            }

            return $title;
        }, 10, 2);
        
        add_filter( 'body_class', function($classes) use ($pageTemplate) {
            $classes[] = sanitize_title($pageTemplate);
            $classes[] = sanitize_title(str_replace(array('.php'),array(''), $pageTemplate));
            $classes[] = sanitize_title(str_replace(array('.php','templates'),array('','template'), $pageTemplate));

            return $classes; 
        });

        $pageTemplate = apply_filters('si_get_page_template', $pageTemplate);

        if($pageTemplate != null){
            $templateDir = get_stylesheet_directory();

            if(file_exists($pageTemplate)){
                //do_action('si/single-page-begin', 'singleListingCtrl');
                include($pageTemplate);
                //do_action('si/single-page-end');
                return;
            }

            $templateDir = get_stylesheet_directory();
            if(strpos($pageTemplate, $templateDir ) !== false) $pageTemplate = str_replace($templateDir,'',$pageTemplate);
            
            if(file_exists($templateDir . '/' . $pageTemplate)){
                include  $templateDir . '/' . $pageTemplate;
            }
        }
        

        if($pageTemplate == null){
            wp_footer();
        }
        do_action('si_page_builder_postrender', $page_id);
    }

    
    public function start_of_template($loadingText = null){
        if($this->page_template_rendered) return;
        if(did_action('si_start_of_template') === 1){
            if($loadingText != null){
                echo('<label class="si-placeholder"  data-ng-show="model==null">' .  __($loadingText,SI) . ' <i class="fal fa-spinner fa-spin"></i></label>');
            }
            echo('<div class="si-content"  ng-cloak si-adaptative-class >');
        }
    }

    public function end_of_template(){
        if($this->page_template_rendered) return;
        if(did_action('si_end_of_template') === 1){
            echo('</div>'); 
        }
    }

    function setupPost(&$postOrId = null){
        global $wp_query, $post;
        $page_id = '';
        // $share_tool = new SiSharing($listing_data);
        // $share_tool->addHook('listing');
        
        // $permalink = $share_tool->getPermalink();
        if(is_string($postOrId) || is_int($postOrId)){
            $post = get_post($postOrId);
            $page_id = $postOrId;
        }
        elseif($post != null){
            $post = $postOrId;
            $page_id = $postOrId->ID;
        }
        //
        if($post == null) return;
        
        
        $page_title = apply_filters('si_page_title', $post->page_title);
        $post->post_title = $page_title;
        $post->post_type = 'page';
        
        setup_postdata( $post );

        if($wp_query != null){
            $wp_query->post = $post;
            $wp_query->posts = array( $post );
            $wp_query->queried_object = $post;
            $wp_query->queried_object_id = $page_id;
            $wp_query->found_posts = 1;
            $wp_query->post_count = 1;
            $wp_query->max_num_pages = 1; 
            $wp_query->is_page = true;
            $wp_query->is_singular = true; 
            $wp_query->is_single = true; 
            $wp_query->is_attachment = false;
            $wp_query->is_archive = false; 
            $wp_query->is_category = false;
            $wp_query->is_tag = false; 
            $wp_query->is_tax = false;
            $wp_query->is_author = false;
            $wp_query->is_date = false;
            $wp_query->is_year = false;
            $wp_query->is_month = false;
            $wp_query->is_day = false;
            $wp_query->is_time = false;
            $wp_query->is_search = false;
            $wp_query->is_feed = false;
            $wp_query->is_comment_feed = false;
            $wp_query->is_trackback = false;
            $wp_query->is_home = false;
            $wp_query->is_embed = false;
            $wp_query->is_404 = false; 
            $wp_query->is_paged = false;
            $wp_query->is_admin = false; 
            $wp_query->is_preview = false; 
            $wp_query->is_robots = false; 
            $wp_query->is_posts_page = false;
            $wp_query->is_post_type_archive = false;
            
        }
    }


    static function getPageRelativePath($page_id, $type, $locale){
        $page = self::findPage($page_id, $type, $locale);
        if($page == null) return $page_id;

        return str_replace( home_url(), "", get_permalink($page));
    }
    static function findPage($page_id,$type,$locale){
        $post = null;

        if(is_numeric($page_id)) {
            $post = get_post($page_id);
            if($post !== null) return $post;
        }

        if(is_string($page_id)){ 
            $post = get_page_by_path($page_id);
            
            if($post !== null) return $post;
            else {
                global $wpdb;

                $postid = $wpdb->get_var( "SELECT ID FROM $wpdb->posts WHERE post_name = '" . $page_id . "'" );
                if($postid != null){
                    $post = get_post($postid);
                    if($post !== null) return $post;
                }
            }
        }

        // try to find the post 
        $layoutPageMap = [
            'listing' => [ 
                'en' => ['listing-details', 'listings/' . $page_id], 
                'fr' => ['details-dune-propriete','details-de-propriete','propriete-details','proprietes/' . $page_id],
                'es' => ['pagina-del-listado', 'propiedades/' . $page_id], 
            ],
            'broker' => ['en' => ['broker-details', 'brokers/' . $page_id, 'team/' . $page_id], 'fr' => ['details-dun-courtier','details-de-courtier','courtier-details','courtiers/' . $page_id,'equipe/' . $page_id]],
            'office' => ['en' => ['listings-details', 'offices/' . $page_id], 'fr' => ['details-dun-bureau','details-de-bureau','bureau-details','bureaux/' . $page_id]],
        ];

        if(isset($layoutPageMap[$type])){
            $layoutList = $layoutPageMap[$type][$locale];
            
            foreach ($layoutList as $layoutPage) {
                if($post !== null) continue;

                $post = get_page_by_path($layoutPage);
                //__c($layoutPage, $post);
            }

        }

        return $post;
    }
        
}

function siShowDirectItemLayer($item, $configs,$layerName="main"){

    $path = apply_filters("si/{$configs->type}/layer/{$layerName}", SI_PLUGIN_DIR . "views/list/{$configs->type}/direct/item-layer.php");

    $styles = [];
    $classes = ['si-layer', 'si-' . $layerName . '-layer'];
    
    $linkButtonLabel = _si_label('Learn more');
    
    if($layerName != 'main' && isset( $configs->list_item_layout->secondary_layer_bg_opacity)){
        $styles[] = '--bg-opacity:' . ($configs->list_item_layout->secondary_layer_bg_opacity/100);
        if($configs->list_item_layout->secondary_layer_bg_opacity < 100){
            $classes[] = 'si-padding';
        }
    }

    $rendererClassName = 'siLayerVar';
    $typedRendererClassName = 'si'.ucfirst($configs->type).'LayerVar';
    if(class_exists($typedRendererClassName)) $rendererClassName = $typedRendererClassName;


    if($rendererClassName::isSimple($configs->list_item_layout->displayed_vars->main)){
        $configs->list_item_layout->displayed_vars->main = $rendererClassName::updateToComplex($configs->list_item_layout->displayed_vars->main, $configs->type);
    }
    if(in_array($configs->type, ['listings','brokers'])){
        if(!$rendererClassName::has('photo',$configs->list_item_layout->displayed_vars->main)){
            $configs->list_item_layout->displayed_vars->main = $rendererClassName::updateToComplex($configs->list_item_layout->displayed_vars->main, $configs->type);
        }
    }


    echo('<div class="' . implode(' ', $classes) . '" style="' . implode(';',$styles) . '">');
    echo('<div class="si-layer-content">');
    
    siShowLayerVars($configs->list_item_layout->displayed_vars->{$layerName},$item, $configs->type);

    if($layerName == 'main'){
        if($configs->type == 'listings'){
            if($layerName=='secondary' && isset($configs->list_item_layout->displayed_vars->secondary) && !siLayerVar::has('flags', $configs->list_item_layout->displayed_vars->secondary)){
                $varRenderer = new siLayerVar('flags');
                $varRenderer->renderWithData($item);
            }
            else if(!siLayerVar::has('flags', $configs->list_item_layout->displayed_vars->main)){
                if(!siLayerVar::has('tags', $configs->list_item_layout->displayed_vars->main)){
                    $varRenderer = new siLayerVar('flags');
                    $varRenderer->renderWithData($item);
                }
            }
        }
    }

    //include $path;
    echo('</div>');
    echo('</div>');
}

function siShowStandardItemLayer($configs,$layerName="main"){

    $path = apply_filters("si/{$configs->type}/layer/{$layerName}", SI_PLUGIN_DIR . "views/list/{$configs->type}/standard/item-layer.php");

    $styles = [];
    $classes = ['si-layer', 'si-' . $layerName . '-layer'];
    
    $linkButtonLabel = _si_label('Learn more');
    
    if($layerName != 'main' && isset( $configs->list_item_layout->secondary_layer_bg_opacity)){
        $styles[] = '--bg-opacity:' . ($configs->list_item_layout->secondary_layer_bg_opacity/100);
        //if($configs->list_item_layout->secondary_layer_bg_opacity < 100){
        //    $classes[] = 'si-padding';
        //}
    }

    echo('<div class="' . implode(' ', $classes) . '" style="' . implode(';',$styles) . '">');
    echo('<div class="si-layer-content">');
    //include $path;
    
    $rendererClassName = 'siLayerVar';
    $typedRendererClassName = 'si'.ucfirst($configs->type).'LayerVar';
    if(class_exists($typedRendererClassName)) $rendererClassName = $typedRendererClassName;
    
    if($rendererClassName::isSimple($configs->list_item_layout->displayed_vars->main)){
        // error_log($configs->type . ' is simple');
        $configs->list_item_layout->displayed_vars->main = $rendererClassName::updateToComplex($configs->list_item_layout->displayed_vars->main, $configs->type);
    }
    else if($configs->type != 'cities' && !$rendererClassName::has('photo',$configs->list_item_layout->displayed_vars->main)){
        $configs->list_item_layout->displayed_vars->main = $rendererClassName::updateToComplex($configs->list_item_layout->displayed_vars->main, $configs->type);
    }

    siShowLayerVars($configs->list_item_layout->displayed_vars->{$layerName},null, $configs->type);

    if($layerName == 'main'){
        if($configs->type == 'listings'){
            if($layerName=='secondary' && isset($configs->list_item_layout->displayed_vars->secondary) && !siLayerVar::has('flags', $configs->list_item_layout->displayed_vars->secondary)){
                $varRenderer = new siLayerVar('flags');
                $varRenderer->render();
            }
            // Check listings mandatory vars: flags or tags must be present
            else if(!siLayerVar::has('flags', $configs->list_item_layout->displayed_vars->main)){
                if(!siLayerVar::has('tags', $configs->list_item_layout->displayed_vars->main)){
                    $varRenderer = new siLayerVar('flags');
                    $varRenderer->render();
                }
            }
        }
    }

    echo('</div>');
    echo('</div>');
}

function siShowLayerVars($varList, $data=null, $type=null){
    $rendererClassName = 'siLayerVar';
    $typedRendererClassName = 'si'.ucfirst($type).'LayerVar';
    if(class_exists($typedRendererClassName)) $rendererClassName = $typedRendererClassName;
    
    foreach ($varList as $varItem) {
        $varRenderer = new $rendererClassName($varItem,false);
        if($data == null) $varRenderer->render();
        if($data != null) $varRenderer->renderWithData($data);
    }
}
