<?php

class SourceImmoTemplateManager {

  function __construct(){
    

      add_action('template_redirect', [$this, 'template_redirect']);
  }

	function template_redirect(){
        $ref_number = get_query_var( 'ref_number' );
        
        //error_log('SourceImmoTemplateManager:template_redirect with ' . $ref_number);

        if ( $ref_number ) {
          $type = get_query_var( 'type' );
          $mode = get_query_var( 'mode' );
          $lang = get_query_var( 'lang' );
    
          //error_log('SourceImmoTemplateManager:template_redirect with ' . "{$ref_number} {$type} {$mode} {$lang} ");

          if($mode == 'shortcut'){
            // redirect to long
            do_action('si/shortcut_redirect',$type, $ref_number,$lang );

            exit();
          }
          else{
            if($mode == 'print'){
              $mode = '_print';
            }
            else{
              $mode = '_detail_template';
            }
    
            if( $this->custom_page_exists($ref_number, $pathToRedirect) ){ 
              //error_log('SourceImmoTemplateManager:template_redirect redirect to custom page ' . $pathToRedirect);
              wp_redirect($pathToRedirect); 
              exit();
            }
    
            add_filter( 'template_include', array($this, 'include_' . $type . $mode), 5);
          }
        }
  }
  
  function custom_page_exists($ref_number,&$pathToRedirect){
    $path = $_SERVER['REQUEST_URI'];
    $refIndex = strpos($path, $ref_number);
    if($refIndex === false) return false;
    $currentLocale = si_get_locale();

    $basePath = substr($path, 0, $refIndex);


    $pageVariations = [];
    $pageVariations[] = $basePath;
    $pageVariations[] = str_replace($currentLocale . '/','', $basePath);

    foreach ($pageVariations as $path) {
      if(get_page_by_path($path)){ 
        $pathToRedirect = $basePath;
        return true;
      }
    }
    return false;
  }


  
  function include_listings_detail_template($template){

    global $wp_query;
    // prevent themes or plugins to trap these state for some reason
    $wp_query->is_page = true;
    $wp_query->is_home = false;
    $wp_query->is_archive = false;
    $wp_query->is_search = false;


    $ref_number = get_query_var( 'ref_number' );
    global $permalink, $post,$listing_data;
    
    // load data
    global $listing_data;
    global $siCurrentModel;
    $rawModel = SourceImmoApi::get_listing_data($ref_number);
    $siCurrentModel = $listing_data = json_decode( $rawModel);

    if($listing_data != null){
      do_action('si_listing_detail_begin');

      // hook to sharing tools
      $share_tool = new SiSharing($listing_data);
      $share_tool->addHook('listing');
      $permalink = $share_tool->getPermalink();

      if(isset($_GET['view'])){
        $permalink = $permalink . '?view=' . $_GET['view'];
      }

      add_filter('si_page_title', function($title) use ($share_tool){
        return $share_tool->title();
      });

      if(!isset($post)) $post = json_decode('{}');
      //$post->type = 'page';
      $post->permalink = $permalink;

      // add hook for permalink
      add_filter('the_permalink', function($url){
        global $permalink;
        return $permalink;
      });
      add_filter('get_canonical_url', function($url) use ($permalink){
        $result = get_site_url() . $permalink;
        return $result;
      });

      add_action('the_post', function($post_object){
        global $permalink;
        $post_object->permalink = $permalink;
        $post_object->post_title = '';
      });

      // Set bootstrap target element to <html>
      add_filter('si/bootstrap_target_element', function($element){ return 'html'; });

      // Add hook to data
      //$listing_data = apply_filters(hook_from_key('listing','single'), $listing_data);
      return SI_PLUGIN_DIR . 'views/single/listings.php';

      SourceImmo::view('single/listings', array('ref_number'=>$ref_number, 'data' => $listing_data, 'permalink' => $permalink));
      die();
    }
    else{
      SourceImmo::current()->redirect_not_found('listings');
    }
  }

  function include_listings_print(){
    $ref_number = get_query_var( 'ref_number' );
    
    global $wp_query;
    // prevent themes or plugins to trap these state for some reason
    $wp_query->is_page = true;
    $wp_query->is_home = false;
    $wp_query->is_archive = false;
    $wp_query->is_search = false;

    // load data
    $model = json_decode(SourceImmoApi::get_listing_data($ref_number));
    if($model != null){
      $brokerIds = array_map(function($item){
        return $item->ref_number;
      }, $model->brokers);
      $brokers = json_decode(SourceImmoApi::get_broker_list($brokerIds));

      // Load global dictionary
      $global_lexicon = SourceImmoApi::get_dictionary();
      // Assign broker list
      $model->brokers = $brokers;
      
      do_action('si/listing/print', $model);

      $this->remove_theme_styles();

      header('http/1.0 200 found');
      global $dictionary;
      // hook to sharing tools

      // $share_tool = new SiSharing($model);
      // $share_tool->addHook('listing');

      $listingWrapper = new SourceImmoListingsResult();
      
      $dictionary = new SourceImmoDictionary($model->dictionary);
      $dictionary->addEntries($global_lexicon);

      $listingWrapper->preprocess_item($model);
      $listingWrapper->extendedPreprocess($model);
      $model = apply_filters('si_listing_print_post_process', $model);

      $model->permalink = SourceImmoListingsResult::buildPermalink($model, SourceImmo::current()->get_listing_permalink());

      if(isset($_GET['view'])){
        $model->permalink = $model->permalink . '?view=' . $_GET['view'];
      }
      $protocol = (!empty($_SERVER['HTTPS']) && $_SERVER['HTTPS'] !== 'off' || $_SERVER['SERVER_PORT'] == 443) ? "https://" : "http://";
      $model->tiny_url = SourceImmoTools::get_tiny_url($protocol . $_SERVER['HTTP_HOST'] . $model->permalink);
      
      
      $lRootUrl = $this->get_root_url();
      $locale = si_get_locale();
      $lRootUrl = str_replace($locale . '/','',$lRootUrl);
      $plugin_url = SI_PLUGIN_URL;
      if($lRootUrl != '/'){
        $plugin_url = str_replace($lRootUrl, "/", $plugin_url);
      }
      wp_enqueue_style('listing-print', $plugin_url . 'styles/print.min.css', null, filemtime(SI_PLUGIN_DIR . '/styles/print.min.css'));
      wp_enqueue_style( 'fontawesome6', 'https://kit.fontawesome.com/cba8c40f52.css' );
      do_action('si/listing/print:begin');

      $filePath = SourceImmo::file('single/listings_print');
      if($filePath != null){
        include $filePath;
      }
      die();
    }
    else{
      header('http/1.0 404 not found');

      SourceImmo::view('single/listings_404', array());
      die();
    }
  }
  
  function include_brokers_detail_template(){
    global $wp_query;
    // prevent themes or plugins to trap these state for some reason
    $wp_query->is_page = true;
    $wp_query->is_home = false;
    $wp_query->is_archive = false;
    $wp_query->is_search = false;

    $ref_number = get_query_var( 'ref_number' );
    global $siCurrentModel;
    global $broker_data,$post;
    // load data
    $siCurrentModel = $broker_data = json_decode(SourceImmoApi::get_broker_data($ref_number));

    if($broker_data != null){
      header('http/1.0 200 found');
      do_action('si_broker_detail_begin');

      // hook to sharing tools
      $share_tool = new SiSharing($broker_data);
      $share_tool->addHook('broker');
      
      $permalink = $share_tool->getPermalink();
      
      if(isset($_GET['view'])){
        $permalink = $permalink . '?view=' . $_GET['view'];
      }

      add_filter('si_page_title', function($title) use ($share_tool){
        return $share_tool->title();
      });
      if($post != null) $post->permalink = $permalink;

      // add hook for permalink
      add_filter('the_permalink', function($url){
        global $permalink;
        return $permalink;
      });
      add_action('the_post', function($post_object){
        global $permalink;
        $post_object->permalink = $permalink;
        $post_object->post_title = '';
      });
      add_filter('get_canonical_url', function($url) use ($permalink){
        $result = get_site_url() . $permalink;
        return $result;
      });

      // Set bootstrap target element to <html>
      add_filter('si/bootstrap_target_element', function($element){ return 'html'; });

      $broker_data = apply_filters(hook_from_key('broker','single'), $broker_data);

      SourceImmo::view('single/brokers', array('ref_number'=>$ref_number, 'data' => $broker_data));
      die();
    }
    else{
      SourceImmo::current()->redirect_not_found('brokers');
    }
  }

  function include_cities_detail_template(){
    global $wp_query;
    // prevent themes or plugins to trap these state for some reason
    $wp_query->is_page = true;
    $wp_query->is_home = false;
    $wp_query->is_archive = false;
    $wp_query->is_search = false;
    
    $ref_number = get_query_var( 'ref_number' );
    global $permalink, $post,$city_data;

    // load data
    // global $city_data;
    $city_data = SourceImmoApi::get_city_data($ref_number)->items[0];
    //__c($city_data);

    global $siCurrentModel;
    $siCurrentModel = $city_data;
    // $city_data = new SourceImmoListingsResult($city_data);
    if($city_data != null){
      header('http/1.0 200 found');
      do_action('si_city_detail_begin');
      // hook to sharing tools
      $share_tool = new SiSharing($city_data);
      $share_tool->addHook('city');
      
      $permalink = $share_tool->getPermalink();
      
      if(isset($_GET['view'])){
        $permalink = $permalink . '?view=' . $_GET['view'];
      }

      if(isset($post)) $post->permalink = $permalink;
      add_filter('si_page_title', function($title) use ($share_tool){
        return $share_tool->title();
      });
      
      // add hook for permalink
      // add_filter('the_permalink', function($url){
      //   global $permalink;
      //   return $permalink;
      // });
      add_action('the_post', function($post_object){
        global $permalink;
        $post_object->permalink = $permalink;
        $post_object->post_title = '';
      });
      add_filter('get_canonical_url', function($url) use ($permalink){
        $result = get_site_url() . $permalink;
        return $result;
      });

      // Set bootstrap target element to <html>
      add_filter('si/bootstrap_target_element', function($element){ return 'html'; });

      $city_data = apply_filters(hook_from_key('city','single'), $city_data);
      SourceImmo::view('single/cities', array('ref_number'=>$ref_number, 'data' => $city_data, 'permalink' => null));      
      die();
    }
    else{
      SourceImmo::current()->redirect_not_found('cities');
    }
  }

  function include_offices_detail_template(){
    global $wp_query;
    // prevent themes or plugins to trap these state for some reason
    $wp_query->is_page = true;
    $wp_query->is_home = false;
    $wp_query->is_archive = false;
    $wp_query->is_search = false;

    $ref_number = get_query_var( 'ref_number' );
    global $office_data,$post;
    // load data
    global $siCurrentModel;
    $dataJson = SourceImmoApi::get_office_data(strtoupper($ref_number));
    $siCurrentModel = $office_data = json_decode($dataJson);
    
    if($office_data != null){
      header('http/1.0 200 found');
      do_action('si_office_detail_begin');

      // hook to sharing tools
      $share_tool = new SiSharing($office_data);
      $share_tool->addHook('office');
      $permalink = $share_tool->getPermalink();

      if(isset($_GET['view'])){
        $permalink = $permalink . '?view=' . $_GET['view'];
      }

      if($post != null) $post->permalink = $permalink;

      // add hook for permalink
      add_filter('the_permalink', function($url){
        global $permalink;
        return $permalink;
      });
      add_action('the_post', function($post_object){
        global $permalink;
        $post_object->permalink = $permalink;
        $post_object->post_title = '';
      });
      add_filter('get_canonical_url', function($url) use ($permalink){
        $result = get_site_url() . $permalink;
        return $result;
      });

      // Set bootstrap target element to <html>
      add_filter('si/bootstrap_target_element', function($element){ return 'html'; });


      $office_data = apply_filters(hook_from_key('office','single'), $office_data);
      SourceImmo::view('single/offices', array('ref_number'=>$ref_number, 'data' => $office_data, 'permalink' => null));      
      die();
    }
    else{
      error_log('SourceImmoTemplateManager:include_offices_detail_template office data not found');
      error_log('SourceImmoTemplateManager:include_offices_detail_template::json_err = ' . json_last_error_msg());
      error_log($dataJson);
      
      SourceImmo::current()->redirect_not_found('offices');
    }
  }

  function include_agencies_detail_template(){
    global $wp_query;
    // prevent themes or plugins to trap these state for some reason
    $wp_query->is_page = true;
    $wp_query->is_home = false;
    $wp_query->is_archive = false;
    $wp_query->is_search = false;

    $ref_number = get_query_var( 'ref_number' );
    global $agency_data,$post;
    // load data
    
    global $siCurrentModel;
    $siCurrentModel = $agency_data = json_decode(SourceImmoApi::get_agency_data(strtoupper($ref_number)));
    
    if($agency_data != null){
      header('http/1.0 200 found');
      do_action('si_agency_detail_begin');

      // hook to sharing tools
      $share_tool = new SiSharing($agency_data);
      $share_tool->addHook('agency');
      $permalink = $share_tool->getPermalink();

      if(isset($_GET['view'])){
        $permalink = $permalink . '?view=' . $_GET['view'];
      }

      if($post != null) $post->permalink = $permalink;

      // add hook for permalink
      add_filter('the_permalink', function($url){
        global $permalink;
        return $permalink;
      });
      add_action('the_post', function($post_object){
        global $permalink;
        $post_object->permalink = $permalink;
        $post_object->post_title = '';
      });
      add_filter('get_canonical_url', function($url) use ($permalink){
        $result = get_site_url() . $permalink;
        return $result;
      });
      
      // Set bootstrap target element to <html>
      add_filter('si/bootstrap_target_element', function($element){ return 'html'; });


      $agency_data = apply_filters(hook_from_key('agency','single'), $agency_data);
      SourceImmo::view('single/agencies', array('ref_number'=>$ref_number, 'data' => $agency_data, 'permalink' => null));      
      die();
    }
    else{
      SourceImmo::current()->redirect_not_found('agencies');
    }
  }

  
  function remove_theme_styles(){
    add_action('wp_enqueue_scripts',[$this,'unload_theme_resources']);
  }
  public function unload_theme_resources(){
    $themeData       = wp_get_theme();
    $themeTemplate = strtolower($themeData->get_template());
    $themeStylesheet =  strtolower($themeData->get_stylesheet());
    $themeStylesheet = str_replace(
                            ['-child','-theme'],
                            ['',''],
                            $themeStylesheet
                        );
  
    
    $styleVariations = [
        'style',
        'stylesheet',
        'print-style',
        'print-stylesheet',
    ];
    
    $baseVariations = [
      $themeStylesheet,
      $themeStylesheet . '-parent',
    ];
    if($themeTemplate != $themeStylesheet){
      $baseVariations[] = $themeTemplate;
    }

    foreach ($baseVariations as $base) {
      wp_dequeue_style( $base );
      wp_deregister_style( $base );
      
      foreach ($styleVariations as $var) {
        wp_dequeue_style( $base . '_' . $var );
        wp_deregister_style( $base . '_' . $var );

        wp_dequeue_style( $base . '-' . $var );
        wp_deregister_style( $base . '-' . $var );
      }
    }
    
  }
  
  private $_root_url = [];

  function get_root_url($lang=null){
    
    if(isset($this->_root_url[$lang])) return $this->_root_url[$lang];

    $lRootUrl = get_bloginfo('url');
    $lRootUrl = str_replace(['http://', 'https://'],'', $lRootUrl);
    
    if(class_exists('Sitepress')){
      global $sitepress;
      $lTwoLetterLocale = $lang!=null ? $lang : si_get_locale();
      $localeBaseUrl = $sitepress->language_url( $lTwoLetterLocale );
      $localeBaseUrl = str_replace(['http://', 'https://'],'', $localeBaseUrl);
      $localeBaseUrl = trim($localeBaseUrl,'/');
      
      if($localeBaseUrl != $lRootUrl && strpos($localeBaseUrl,'?')===false) $lRootUrl = $localeBaseUrl;
    }
    else if(function_exists('pll_home_url')){
      $lTwoLetterLocale = $lang!=null ? $lang : si_get_locale();
      $localeBaseUrl = pll_home_url($lang);
      $localeBaseUrl = str_replace(['http://', 'https://'],'', $localeBaseUrl);
      $localeBaseUrl = trim($localeBaseUrl,'/');
      if($localeBaseUrl != $lRootUrl && strpos($localeBaseUrl,'?')===false) $lRootUrl = $localeBaseUrl;

      $langPosInUrl = strpos($lRootUrl,$lTwoLetterLocale);
      if($langPosInUrl !== false){
        $lRootUrl = substr($lRootUrl,0, $langPosInUrl) . $lTwoLetterLocale;
      }

      
    }
    
    
    if(strpos($lRootUrl,'/') !== false){
      $lRootUrl = substr($lRootUrl,strpos($lRootUrl,'/')) . '/';
    }
    else{
      $lRootUrl = '/';
    }

    $this->_root_url[$lang] = $lRootUrl;
    return $lRootUrl;
  }
}