
siApp
.factory('$siSearchContext', ['$rootScope','$filter', function $siSearchContext($rootScope,$filter){
    let lToday = new Date().round();    // save today
    let lNow = new Date();

    class SearchContextFactory {

        constructor(){
            this.initializeData();
            this.registerEvents();
        }
        

        initializeData(){
            this.slaves = [];

            this.templates = {};
            this.templates.main_filters = {
                listings : {
                    'RES' : {field: 'market_codes', operator: 'array_contains', value: 'RES', type: 'market'},
                    'COM' : {field: 'market_codes', operator: 'array_contains', value: 'COM', type: 'market'},
                    'for-sale' : {field: 'for_sale_flag', operator: 'equal', value: true, type: 'flag'},
                    'for-rent' : {field: 'for_rent_flag', operator: 'equal', value: true, type: 'flag'}
                }
            }
            // listing states
            this.templates.listing_states = [
                {
                    key: 'sale',
                    caption: 'For sale'.siTranslate(), 
                    filter : {field: 'for_sale_flag', operator: 'equal', value: true}
                },
                {
                    key: 'rent',
                    caption: 'For rent'.siTranslate(),
                    filter : {field: 'for_rent_flag', operator: 'equal', value: true}
                }            
            ]

            // listing states
            this.templates.listing_flags = [
                {
                    key: 'sold',
                    caption : 'Sold',
                    filter : {field: 'status_code', operator: 'not_equal_to', value: 'AVAILABLE'}
                },
                {
                    key: 'available',
                    caption : 'On the market',
                    filter : {field: 'status_code', operator: 'equal', value: 'AVAILABLE'}
                },
                {
                    key: 'open_house',
                    caption: 'Open house',
                    filter : {field: 'open_houses[0].end_date', operator: 'greater_or_equal_to', value: 'udf.now()'}
                },
                {
                    key: 'foreclosure',
                    caption: 'Foreclosure',
                    filter : {field: 'price.foreclosure', operator: 'equal', value: true}
                },
                {
                    key: 'virtual_tour',
                    caption: 'With virtual tour',
                    filter : {field: 'virtual_tour_flag', operator: 'equal', value: true}
                },
                {
                    key: 'video',
                    caption: 'With video',
                    filter : {field: 'video_flag', operator: 'equal', value: true}
                }
            ];

            this.templates.listing_tags = [];
            this.templates.broker_tags = [];
            this.templates.office_tags = [];

            // listing ages or timespan filters
            this.templates.listing_ages = [
                {
                    key: '24h',
                    caption: '24 hours'.siTranslate(), 
                    filter : {field: 'contract.start_date', operator: 'greater_than', value: lNow.addDays(-1).toJSON()}
                },
                {
                    key: '48h',
                    caption: '48 hours'.siTranslate(), 
                    filter : {field: 'contract.start_date', operator: 'greater_than', value: lNow.addDays(-2).toJSON()}
                },
                {
                    key: '7d',
                    caption: '7 days'.siTranslate(), 
                    filter : {field: 'contract.start_date', operator: 'greater_than', value: lToday.addDays(-7).toJSON()}
                },
                {
                    key: '2w',
                    caption: '2 weeks'.siTranslate(), 
                    filter : {field: 'contract.start_date', operator: 'greater_than', value: lToday.addDays(-14).toJSON()}
                },
                {
                    key: '1m',
                    caption: '1 month'.siTranslate(),
                    filter : {field: 'contract.start_date', operator: 'greater_than', value: lToday.addMonths(-1).toJSON()}
                },
                {
                    key: '3m',
                    caption: '3 months'.siTranslate(),
                    filter : {field: 'contract.start_date', operator: 'greater_than', value: lToday.addMonths(-3).toJSON()}
                },
                {
                    key: '6m',
                    caption: '6 months'.siTranslate(),
                    filter : {field: 'contract.start_date', operator: 'greater_than', value: lToday.addMonths(-6).toJSON()}
                },
                {
                    key: '1y',
                    caption: '1 year'.siTranslate(),
                    filter : {field: 'contract.start_date', operator: 'greater_than', value: lToday.addYears(-1).toJSON()}
                },
                {
                    key: '',
                    caption : 'More than a year'.siTranslate(),
                    filter : {field: 'contract.start_date', operator: 'greater_than', value: ''}
                },
            ];

            this.templates.market_types = [
                {key: 'RES', caption: 'Residential'.siTranslate(), filter: {field: 'market_codes', operator: 'array_contains', value: 'RES'} },
                {key: 'COM', caption: 'Commercial'.siTranslate(), filter: {field: 'market_codes', operator: 'array_contains', value: 'COM'} }
            ]

            // listing land areas
            this.templates.land_areas = Array.from(Array(3)).map(function($e,$i){
                return { caption: '{0} sqft'.siTranslate().siFormat(5000 * ($i+1)), value: 5000 * ($i+1)}
            });
            this.templates.land_areas = this.templates.land_areas.concat(Array.from(Array(10)).map(function($e,$i){
                return { caption: '{0} sqft'.siTranslate().siFormat(20000 * ($i+1)), value: 20000 * ($i+1)}
            }));


            // listing available areas
            this.templates.available_areas = Array.from(Array(6)).map(function($e,$i){
                return { caption: '{0} sqft'.siTranslate().siFormat(250 * ($i+2)), value: 200 * ($i+2)}
            });
            this.templates.available_areas = this.templates.available_areas.concat(Array.from(Array(9)).map(function($e,$i){
                return { caption: '{0} sqft'.siTranslate().siFormat($filter('number')(1000 * ($i+2))), value: 1000 * ($i+2)}
            }));
            this.templates.available_areas = this.templates.available_areas.concat(Array.from(Array(5)).map(function($e,$i){
                return { caption: '{0} sqft'.siTranslate().siFormat($filter('number')(2000 * ($i+6))), value: 2000 * ($i+6)}
            }));
            this.templates.available_areas = this.templates.available_areas.concat(Array.from(Array(6)).map(function($e,$i){
                return { caption: '{0} sqft'.siTranslate().siFormat($filter('number')(5000 * ($i+5))), value: 5000 * ($i+5)}
            }));
            this.templates.available_areas = this.templates.available_areas.concat(Array.from(Array(4)).map(function($e,$i){
                return { caption: '{0} sqft'.siTranslate().siFormat($filter('number')(10000 * ($i+6))), value: 10000 * ($i+6)}
            }));
            this.templates.available_areas = this.templates.available_areas.concat(Array.from(Array(9)).map(function($e,$i){
                return { caption: '{0} sqft'.siTranslate().siFormat($filter('number')( 50000 * ($i+2)) ), value: 50000 * ($i+3)}
            }));

            // this.templates.bedroomSuggestions = {buildFrom: { buildCountedList : ['{0}+ bedrooms',5]}}
            // this.templates.bathroomSuggestions = {buildFrom: { buildCountedList : ['{0}+ bathrooms',5]}}
            // this.templates.garageSuggestions = {buildFrom: { buildCountedList : ['{0}+ garages',5]}}
            // this.templates.parkingSuggestions = {buildFrom: { buildCountedList : ['{0} spaces or +',5,1,'1 space or +']}}
            
            this.templates.bedroomSuggestions = this.buildCountedList('{0}+ bedrooms',5)
            this.templates.bathroomSuggestions = this.buildCountedList('{0}+ bathrooms',5)
            this.templates.garageSuggestions = this.buildCountedList('{0}+ garages',5)
            this.templates.parkingSuggestions = this.buildCountedList('{0} spaces or +',5,1,'1 space or +')
            
            // listing attributes
            this.templates.listing_attributes = [
                {
                    key: 'pool',
                    caption: 'Pool', 
                    field: 'attributes.POOL',
                    tags: ['residential']
                },
                {
                    key: 'fireplace',
                    caption: 'Fireplace', 
                    field: 'attributes.HEART_STOVE',
                    tags: ['residential']
                },
                {
                    key: 'elevator',
                    caption: 'Elevator', 
                    field: 'attributes.ELEVATOR'
                },
                {
                    key: 'garage',
                    caption: 'Garage', 
                    field: 'attributes.GARAGE'
                },
                {
                    key: 'waterfront',
                    caption: 'Water front', 
                    field: 'attributes.WATER_FRONT',
                    tags: ['residential']
                },
                {
                    key: 'panoramicview',
                    caption: 'Panoramic view', 
                    field: 'attributes.PANORAMIC_VIEW',
                    tags: ['residential']
                }
            ]
            
        }

        buildCountedList($labelFormat, $iteration, $step, $singleLabelFormat){
            //console.log('buildCountedList', $name,$labelFormat, $iteration, $step, $singleLabelFormat)
            $step = $step == undefined ? 1 : $step;
            $singleLabelFormat = $singleLabelFormat == undefined ? $labelFormat : $singleLabelFormat;
    
            const fnNewItem = ($i) => {
                return {
                    value: $i,
                    label : ($i == 1 ? $singleLabelFormat : $labelFormat).siTranslate().siFormat($i),
                    caption : ($i == 1 ? $singleLabelFormat : $labelFormat).siTranslate().siFormat($i)
                }
            }
    
            return Array.from(Array($iteration))
                                .map(($e,$i) => {
                                    return fnNewItem( ($i + 1) * $step );
                                });
        }

        extendTo($target){
            this.slaves.push($target);
    
            this.updateLists($target);
        }

        updateSlaves(){
            this.slaves.forEach(($slave) => {
                this.updateLists($slave);
            });
        }

        updateLists($target){
            //$scope.buildDynamicLists();
    
            Object.keys(this.templates)
            .forEach(function($att){
                // make sure it's a clean slate
                delete $target[$att];
            })
    
            Object.keys(this.templates)
            .filter(($att) => {
                // filter "slaves" attribute
                if(['slaves','main_filters'].includes($att)) return false;             
    
                // filter local function
                if(typeof(this.templates[$att]) == 'function') return false;    
    
                // Bailout when viewFilters is null
                if($target.viewFilters == null) return true;            
    
                // Bailout when filtering with viewFilters returns false
                if(this.filterListByKey($att, $target.viewFilters) == false) return false;
                
                return true;
            })
            .forEach(($att) => {
                // Copy original
                const lAttr = angular.copy(this.templates[$att]);
    
                if(lAttr.buildFrom != undefined){
                    const fnName = Object.keys(lAttr.buildFrom)[0];
                    const fnParams = lAttr.buildFrom[fnName];
                    fnParams.splice(0,0,$att);
                    $target[$att] = this[fnName].apply(null,fnParams);
                }
                else{
                    $target[$att] = lAttr.filter(($item) => {
                        if($target.viewFilters == null) return true;
                        return this.filterListItem($item, $target.viewFilters);
                    })
                }
            });

        }

        registerEvents(){
            $rootScope.$on('si/viewMeta:change', ($event, $alias, $viewMeta) => {
                // respawn list based on meta 
                this.slaves
                    .filter( ($s) => {return $s.alias == $alias})
                    .forEach(($slave) => {
                        this.updateLists($slave);
                    });
            })
        }

        
        filterListByKey($att, $viewFilters){
            return $viewFilters.every( ($f) => {
                if($f.field=='StatusCode' && $f.value== 'SOLD'){
                    return !['listing_states'].includes($att);
                }

                if(['SellPrice','LeasePrice'].includes($f.field)){
                    return !['listing_states'].includes($att);
                }

                return true;
            })

        }

        filterListItem($item, $viewFilters){
            // all filter must return true to display the item
            return $viewFilters.every(($f) => {

                // search for SOLD filter and display only if not found
                if($f.field=='StatusCode' && $f.value== 'SOLD'){
                    // item key must NOT be one of these
                    return !['sold','available','open_house','foreclosure'].includes($item.key);
                }

                // search for AVAILABLE filter and display only if not found
                if($f.field=='StatusCode' && $f.value== 'AVAILABLE'){
                    // item key must NOT be one of these
                    return !['sold','available'].includes($item.key);
                }
                if(['SellPrice','LeasePrice'].includes($f.field)){
                    return !['sold','available'].includes($item.key);
                }


                if($f.field=='MarketCodes' && $f.value=='COM'){
                    return !['pool','fireplace','waterfront','panoramicview'].includes($item.key);
                }

                return true;
            });
        }
    }

    return new SearchContextFactory();

}]);


siApp
.factory('$siFilters', ['$rootScope','$q','$siApi','$siUtils','$timeout', '$siSearchContext', '$siConfig',
function $siFilters($rootScope,$q,$siApi,$siUtils,$timeout,$siSearchContext, $siConfig){
    class FilterFactory {
        constructor(alias){
            this._updatePending = false;
            this.state_loaded = false;
            this.alias = alias;
            this.result_url = null;
            this.data = {};
            this.sort_fields = [];
            this.query_text = null;
            this.main_filter = null;
            this.max_item_count = null;

            this._events_listener = {};
            
            this.$ready = false;
            this.$readyPendings = [];

        }

        init($configs = null){
            if($configs != null){
                this.$configs = $configs;
            }
            //console.log('$siFilter/init', this.alias);

            return $q( ($resolve) => {
                this.getConfigs().then(configs => {
                    this.dataType = configs.type;

                    
                    if(this.state_loaded != true){ 
                        this.data = this.getDefaultFilters();
                        this.loadState();
                        this.loadStateFromQueryString();
                    }
                    
                    if(this.isNotDefault()){
                        this.update(false, false);
                    }
                
                    this.resolvePendingReady();
                    $resolve();

                    // $timeout(_ => {
                    //     if(this.$ready !== true){
                    //         this.$ready.resolve();                            
                    //     } 
                    //     this.$ready = true;
                    //     $resolve();
                    // }, 250);
                })
            });
        }
        getConfigs(){
            if(!isNullOrUndefined(this.$configs)) return $q.resolve(this.$configs);
            return $siConfig.getList(this.alias).then( $configs => {
                this.$configs = $configs;
                return this.$configs;
            })
        }

        isReady(){
            if(this.$ready === true) return $q.resolve();
            const pending = $q.defer();
            this.$readyPendings.push(pending);
            return pending.promise;
        }
        resolvePendingReady(){
            this.$ready = true;
            this.$readyPendings.forEach( p => {
                p.resolve();
            });

            this.$readyPendings = [];
        }

        addGeoFilter (){
            $q( ($resolve,$reject) => {
                if(this.data.location != null){
                    this.data.location = null;
                    this.update();
                    $resolve();
                }
                else{
                    navigator.geolocation.getCurrentPosition( ($position) => {
                        //console.log($position);
                        this.data.location = {
                            latitude: $position.coords.latitude,
                            longitude: $position.coords.longitude,
                            distance : 5000 // set radius to 5Km
                        };
    
                        $resolve();
                    });
                }
            }).then(() => {
                //this.saveState();
                this.update();
                //this.trigger('update');
            });
            
        }

        getDefaultFilters(){
            const dataMap = {
                listings : {
                    keyword : '',
                    min_price: null,
                    max_price: null,
                    transaction_type: null,
                    attributes:  [],
                    states:[],
                    tags: [],
                    location_countries:[],
                    location_states:[],
                    location_regions: [],
                    location_cities: [],
                    building_categories: [],
                    categories: [],
                    subcategories: [],
                    parkings: null,
                    contract: null,
                    bedrooms: null,
                    bathrooms: null,
                    land_min: null,
                    land_max: null,
                    available_min: null,
                    available_max: null,
                    favorites: null,
                },
                brokers: {
                    keyword : '',
                    tags: [],
                    location_countries:[],
                    location_states:[],
                    location_regions: [],
                    location_cities: [],
                    favorites: null,
                    letters: null,
                    licenses: [],
                    offices: [],
                    languages: null,
                    agencies: []
                },
                offices: {
                    keyword : '',
                    tags: [],
                    location_countries:[],
                    location_states:[],
                    location_regions: [],
                    location_cities: [],
                    favorites: null,
                    letters: null,
                    licenses: [],
                    offices: [],
                    languages: null,
                    agencies: []
                },
                agencies: {
                    keyword : '',
                    tags: [],
                    location_countries:[],
                    location_states:[],
                    location_regions: [],
                    location_cities: [],
                    favorites: null,
                    letters: null,
                    licenses: [],
                    offices: [],
                    languages: null,
                    agencies: []
                }
            }
            if(this.dataType == null) return {keyword:'', tags: []};

            return dataMap[this.dataType];
        }
        resetFilters(clearState=true){
            //console.log('filter reset!')
            const dataMap = {
                listings : {
                    keyword : '',
                    min_price: null,
                    max_price: null,
                    transaction_type: null,
                    attributes:  [],
                    states:[],
                    tags: [],
                    location_countries:[],
                    location_states:[],
                    location_regions: [],
                    location_cities: [],
                    building_categories: [],
                    categories: [],
                    subcategories: [],
                    parkings: null,
                    contract: null,
                    bedrooms: null,
                    bathrooms: null,
                    land_min: null,
                    land_max: null,
                    available_min: null,
                    available_max: null,
                    favorites: null,
                },
                brokers: {
                    keyword : '',
                    tags: [],
                    location_countries:[],
                    location_states:[],
                    location_regions: [],
                    location_cities: [],
                    favorites: null,
                    letters: null,
                    licenses: [],
                    offices: [],
                    languages: null,
                    agencies: []
                },
                offices: {
                    keyword : '',
                    tags: [],
                    location_countries:[],
                    location_states:[],
                    location_regions: [],
                    location_cities: [],
                    favorites: null,
                    letters: null,
                    licenses: [],
                    offices: [],
                    languages: null,
                    agencies: []
                },
                agencies: {
                    keyword : '',
                    tags: [],
                    location_countries:[],
                    location_states:[],
                    location_regions: [],
                    location_cities: [],
                    favorites: null,
                    letters: null,
                    licenses: [],
                    offices: [],
                    languages: null,
                    agencies: []
                }
            }
            this.data = dataMap[this.dataType];

            if(clearState){
                this.clearState();
            }
            this.trigger('filterTokenReset');

        }
        hasFilter($fieldname){
            if(this.hasFilters()){
                let lFields = $fieldname
                if(!Array.isArray(lFields)){
                    lFields = [$fieldname];
                }
                
                return lFields.some(($f) => {
                    if(this.data[$f] != undefined){
                        if(Array.isArray(this.data[$f])) return this.data[$f].length > 0;
                        if(!isNaN(this.data[$f])){
                            return this.data[$f] > 0 ;
                        }

                        return this.data[$f] != null;
                    }
                    
                    return false; //this.getFilterByFieldName($f) != null
                });
            }
            else{
                return false;
            }
        }

        sublistHasFilters($parent_code, $list, $selections){
            
            if($selections == null) return false;
            if(!Array.isArray($selections)) return false;
            if($selections.length == 0) return false;


            return $list
                .filter(function($item){
                    return $item.parent == $parent_code;
                })
                .some(function($item){
                    return $selections.includes($item.__$obj_key)
                });
        }

        getFilterByFieldName($fieldname, $group){    
            let lResult = null;
            $group = (typeof $group == 'undefined') ? this.filter_group : $group;

            if($group.filters != null){
                $group.filters.some(function($e){
                    if($fieldname.indexOf('*')>=0){
                        if($e.field.indexOf($fieldname.replace("*",""))>=0){
                            lResult = $e;
                            return true;
                        }
                    }
                    else{
                        if($e.field == $fieldname){
                            lResult = $e;
                            return true;
                        }
                    }
                });
            }

            if(lResult == null && $group.filter_groups != null){
                $group.filter_groups.some(function($group){
                    let lGroupResult = this.getFilterByFieldName($fieldname, $group);
                    if(lGroupResult != null){
                        lResult = lGroupResult;
                        return true;
                    }
                });
            }

            return lResult;
            
        }

        hasFilters(){

            return Object.keys(this.data)
                .filter(($k) => {
                    return !['keyword','location'].includes($k);
                })
                .some(($k) => {
                    if(Array.isArray(this.data[$k])){
                        return this.data[$k].length > 0;
                    }
                    if($k == 'min_price'){
                        return this.data[$k] != null && this.data[$k] > 0;
                    }
                    if($k == 'max_price'){
                        return this.data[$k] != null && this.data[$k] > 0;
                    }
                    return this.data[$k] != null && this.data[$k] != '';
                });
            
        }

        isNotDefault(){
            if(this.hasFilters()) return true;

            if(this.sort_fields.length > 0) return true;
            if(this.data.location != null) return true;

            return false;
        }
        

        trigger(event_key){
            const $params = Array.prototype.slice.call(arguments,1);
            console.log('$siFilters/trigger',this.alias, event_key, $params);
            
            $rootScope.$broadcast(`${this.alias}/${event_key}`, $params);
            return;
            //console.log('$siFilters/trigger', event_key, this._events_listener[event_key]);
            // if(typeof this._events_listener[event_key] == 'undefined') return;
            // const $params = Array.prototype.slice.call(arguments,1);
            // this._events_listener[event_key].forEach(function($e) {return $e.apply(undefined, $params)});
        }
        on(event_key){
            if(typeof this._events_listener[event_key] == 'undefined'){
                this._events_listener[event_key] = [];
            }

            let lResult = {
                _callback :null,
                then : function($fn){
                    this._callback = $fn;
                },
                resolve: function(){
                    //console.log('on resolve',this._callback)
                    this._callback.apply(undefined, arguments);
                }
            }

            this._events_listener[event_key].push(function(){
                lResult.resolve.apply(lResult, arguments);
            });

            return lResult;
        }

        getSearchToken(filters){
            return this.getToken(filters);
        }

        // Direct filter methods
        setPricePart(part){
            //console.log('setPricePart', part);
        }
        setLetter(letter){
            //$fm.data.letters = $letter;
            this.data.letters = letter;
            this.update();
        }

        update(saveState=true, trigger=true){
            // Build filters
            const builder = ({
                listings : () => new ListingFilterBuilder(),
                brokers : () => new BrokerFilterBuilder(),
                offices : () => new OfficeFilterBuilder(),
                agencies : () => new FilterBuilder(),
                cities : () => new FilterBuilder()
            })[this.dataType]()

            if(!this.isNotDefault()){
                //console.log('is default', this.previousData);
                if(saveState) this.clearState();
                if(trigger){
                    this.trigger('filterTokenReset', null);
                }
                return;
            }
            //console.log('not default filter', this.sort_fields, this.isNotDefault());

            
            let filterContainer = {};
            if(this.hasFilters()) {
                filterContainer = builder.with(this.data,this.$context).compile();
            }

            if(this.$configs.limit>0){
                filterContainer.max_item_count = this.$configs.limit;
            }

            // apply sortBy
            if(this.sort_fields.length > 0 && this.sort_fields.filter(function($f) {return !isNullOrEmpty($f.field)}).length > 0){
                //if(lResult == null) lResult = {};
                filterContainer.sort_fields = this.sort_fields.filter(function($f) {return !isNullOrEmpty($f.field)});
            }
            else if(this.$configs.sort != 'auto' &&  !isNullOrEmpty(this.$configs.sort) ){
                //if(lResult == null) lResult = {};

                filterContainer.sort_fields = [{field: this.$configs.sort, desc: this.$configs.sort_reverse}];
            }
            
            // apply query
            if(this.query_text != null){
                filterContainer.query_text = this.query_text;
                filterContainer.filter_group = null;    // clear other filter to avoid interference
            }

            // apply proximity filter
            if(this.data.location != null){
                filterContainer.proximity_filter = this.data.location;
            }

            //console.log(filterContainer);

            // Convert filters to SearchToken
            this.getToken(filterContainer).then( token => {
                if(saveState){
                    this._latestToken = token;
                    this.saveState();
                }
                
                if(trigger){
                    this.trigger('filterTokenChanged', token);
                }
            })
        }

        getToken(filters){
            
            if(filters == null) return $q.resolve(null);
            if(this.alias != null && !this.isNotDefault()) return $q.resolve(null);

            const defer = $q.defer();    
            $siApi.api('utils/search_encode', filters).then(
                (response) =>{
                    defer.resolve(response);
                },
                (rejection) => {
                    defer.reject(rejection);
                }
            );
            return defer.promise;
        }


        saveState(item_key, data){
            let lKey = 'si.' + this.alias + '.{0}';
            //console.log(`saveState(${lKey})`);

            if(item_key == undefined){
                this.state_loaded = false;
                //sessionStorage.setItem(lKey.siFormat('filter_group'), JSON.stringify(this.filter_group));
                if(this.query_text!=null){
                    sessionStorage.setItem(lKey.siFormat('query'), this.query_text);
                }
                sessionStorage.setItem(lKey.siFormat('mainFilter'), JSON.stringify(this.main_filter));
                sessionStorage.setItem(lKey.siFormat('data'), JSON.stringify(this.data));
                sessionStorage.setItem(lKey.siFormat('sort'), JSON.stringify(this.sort_fields));
                if(this._latestToken != null){
                    sessionStorage.setItem(lKey.siFormat('latestSearchToken.' + siCtx.locale), this._latestToken);
                }
            }
            else{
                let lValue = data;
                if(typeof(lValue) == 'object'){
                    lValue = JSON.stringify(lValue);
                }
                sessionStorage.setItem(lKey.siFormat(item_key), lValue);
            }
            

        }
        clearState($item_key){
           
            let lKey = 'si.' + this.alias + '.{0}';

            if($item_key == undefined){
                this.state_loaded = false;
                //sessionStorage.removeItem(lKey.siFormat('filter_group'));
                sessionStorage.removeItem(lKey.siFormat('data'));
                sessionStorage.removeItem(lKey.siFormat('mainFilter'));
                sessionStorage.removeItem(lKey.siFormat('query'));
                sessionStorage.removeItem(lKey.siFormat('st'));
                sessionStorage.removeItem(lKey.siFormat('sort'));
            }
            else{
                sessionStorage.removeItem(lKey.siFormat($item_key));
            }
        }

        loadState($item_key){
            const $key = 'si.' + this.alias + '.{0}';

            if($item_key == undefined && !this.state_loaded){
                //console.trace('filterManager.loadState');
                //const lSessionFilterGroup = sessionStorage.getItem($key.siFormat('filter_group'));
                const lSessionData = sessionStorage.getItem($key.siFormat('data'));
                const lMainFilterData = sessionStorage.getItem($key.siFormat('mainFilter'));
                const lQuery = sessionStorage.getItem($key.siFormat('query'));
                const lSort = sessionStorage.getItem($key.siFormat('sort'));

                //let lSessionSearchKeyword = sessionStorage.getItem($key.siFormat('search-keyword'));
                // if(lSessionFilterGroup != null){
                //     this.filter_group = JSON.parse(lSessionFilterGroup);
                // }

                if(!isNullOrUndefined(lSessionData)){
                    let lData = JSON.parse(lSessionData);
                    angular.merge(this.data, lData); 
                    this._updatePending = true;
                }

                if(!isNullOrUndefined(lMainFilterData)){
                    const lMainFilter = JSON.parse(lMainFilterData);
                    this.main_filter = lMainFilter;
                    this._updatePending = true;
                }

                if(!isNullOrUndefined(lSort)){
                    this.sort_fields = JSON.parse(lSort);
                    this._updatePending = true;
                }

                if(!isNullOrUndefined(lQuery)){
                    this.query_text = lQuery;
                    this.data.keyword = lQuery;
                    this._updatePending = true;
                }
                else{
                    this.data.keyword = '';
                }
                this.state_loaded = true;
            }
            else{
                let lResult = sessionStorage.getItem($key.siFormat($item_key));
                try {
                    lResult = JSON.parse(lResult);
                } catch (error) {
                    
                }
                return lResult;
            }
        }

        loadStateFromQueryString(){
            const queryString = new URLSearchParams(window.location.search);
            const search = queryString.get('si-search');
            if(search != null){
                const searchedData = JSON.parse(atob(search));
                this.data = Object.assign(this.data, searchedData);
                this.update();
                //console.log('loadStateFromQueryString', searchedData);
            }
            

            // const lQueryString = $siUtils.getQueryString();
            // const lFilters = $siUtils.getQueryFilters(lQueryString);
            // if(lFilters != null){
            //     this.data = lFilters;
            //     this.update();
            // }
        }
    }

    class FilterBuilder{

        constructor(){
        }

        with(data, context=null){
            this.$context = context;
            this.sourceData = data;
            return this;
        }

        compile(){
            return {
                filter_group: Object.keys(this.sourceData).reduce( (r, k) => {
                    if(this.sourceData[k] == null) return r;
                    if(Array.isArray(this.sourceData[k]) && this.sourceData[k].length == 0) return r;
                    if(this.sourceData[k] == '') return r;

                    if(this[`compile_${k}`] != undefined){
                        this[`compile_${k}`](r, this.sourceData[k]);
                    }
                    else if(Array.isArray(this.sourceData[k])){
                        r.filters.push({field: k, operator: 'in', value: this.sourceData[k] })
                    }
                    return r;
                },{
                    operator: 'AND',
                    filters: [],
                    filter_groups: []
                })
            }
        }

        clean(){}

        compile_tags(container, tags){
            if(tags == null || tags.length == 0) return;
            const tagFilterGroup = {
                operator: 'OR',
                filters: tags.map(t => ({ field: 'tag_ids', operator: 'has_tag',value: t}))
            }
            container.filter_groups.push(tagFilterGroup);
        }
        compile_favorites(container, value){
            container.filters.push({field: 'ref_number', operator: 'in', value: value});
        }
        
        getFilterMap(){
            return {
                transaction_type: function($value){ 
                                    if ($value=='sale') return [{field: 'for_sale_flag', operator: 'equal', value: true},{field: 'for_rent_flag', operator: 'equal', value: ''}];
                                    if ($value=='rent') return [{field: 'for_rent_flag', operator: 'equal', value: true},{field: 'for_sale_flag', operator: 'equal', value: ''}];
                                    return [{field: 'for_rent_flag', operator: 'equal', value: ''},{field: 'for_sale_flag', operator: 'equal', value: ''}]
                                },
                parkings: {field:'attributes.PARKING', operator:'greater_than'},
                contract: function($value){
                                if($context.listing_ages == null) return null;
                                if($value == null) return {field: 'contract.start_date', operator:'greater_than', value: ''};
                                return $context.listing_ages.find(function($age){ return $age.key == $value}).filter;
                            },
                land_min: {field:'land.dimension.area_sf', operator: 'greater_or_equal_to'},
                land_max: {field:'land.dimension.area_sf', operator: 'less_or_equal_to'},
                available_min: {field:'available_area_sf', operator: 'greater_or_equal_to'},
                available_max: {field:'available_area_sf', operator: 'less_or_equal_to'},
                min_price: function($value){
                                //console.log('min_price filter',$value);
                                return {field: $fm.price_parts.map(function($p) {return 'price.' + $p + '.amount'}), operator: 'greater_or_equal_to', value: $value}
                            },
                max_price: function($value){
                                return {field: $fm.price_parts.map(function($p) {return 'price.' + $p + '.amount'}), operator: 'less_or_equal_to', value: $value}
                            },
                building_categories: function($values){
                                return {field: 'building.category_code', operator: 'in', value: $values};
                            },
                attributes: function($value){
                                if($context.listing_attributes == null) return null;

                                return Object.keys($context.listing_attributes).map(function($k){
                                    const lItem = $context.listing_attributes[$k];
                                    const lValue = $value == null 
                                                    ? ''
                                                    : $value.includes(lItem.field) 
                                                        ? true 
                                                        : '';

                                    return {field: lItem.field, operator : 'equal', value: lValue}
                                });                     
                            },
                states: function($value){
                                if($context.listing_flags == null) return null;

                                const lResult = $context.listing_flags
                                    .map(function($flag){
                                        
                                        const lItem = $flag.filter;
                                        
                                        const lValue = $value == null 
                                                        ? ''
                                                        : $value.includes($flag.key) 
                                                            ? lItem.value 
                                                            : '';

                                        return {field: lItem.field, operator : lItem.operator, value: lValue}
                                    });      

                                
                                return lResult;               
                            },
                tags: function($value){
                            if($value == null) $value = [];
                            return $value.map( v => {
                                return { field: 'tag_ids', operator: 'has_tag',value: v}
                            })
                        },
                categories: {field: 'category_code','operator': 'in'},
                subcategories : {field: 'subcategory_code',operator: 'in'},
                bedrooms: {field: 'main_unit.bedroom_count',operator: 'greater_or_equal_to'},
                bathrooms: {field: 'main_unit.bathroom_count',operator: 'greater_or_equal_to'},
                
                location_countries: {field: 'location.country_code', operator: 'in'},
                location_states: {field: 'location.state_code', operator: 'in'},
                location_regions: {field: 'location.region_code', operator: 'in'},
                location_cities: {field: 'location.city_code', operator: 'in'},
                
                favorites: {field: 'ref_number', operator: 'in'},
                letters: { field: 'last_name',operator : 'starts_with' },
                licenses: {field: 'license_type_code', operator: 'in'},
                offices: {field: 'office_ref_number', operator: 'in'},
                agencies: {field: 'agency.ref_number', operator: 'in'},
                languages: {field:'languages',operator:'array_contains'},
                market_type: {field: 'market_codes', operator: 'array_contains'}
            }
        }
    }

    class ListingFilterBuilder extends FilterBuilder{
        
        compile_min_price(container, value){
            if(value == 0) return;
            if(value == null) return;

            const newFilterGroup = {
                operator: 'OR',
                filters: ['sell','lease'].map(p => ({ field: `price.${p}.amount`, operator: 'greater_or_equal_to',value: value}))
            }
            container.filter_groups.push(newFilterGroup);
            //{field: $fm.price_parts.map(function($p) {return 'price.' + $p + '.amount'}), operator: 'greater_or_equal_to', value: $value}
        }
        compile_max_price(container, value){
            if(value == null) return;

            const newFilterGroup = {
                operator: 'OR',
                filters: ['sell','lease'].map(p => ({ field: `price.${p}.amount`, operator: 'less_or_equal_to',value: value}))
            }
            container.filter_groups.push(newFilterGroup);
        }
        compile_bedrooms(container, value){
            container.filters.push({field: 'main_unit.bedroom_count',operator: 'greater_or_equal_to',value: parseInt(value)});
        }
        compile_bathrooms(container, value){
            container.filters.push({field: 'main_unit.bathroom_count',operator: 'greater_or_equal_to',value: parseInt(value)});
        }

        compile_land_min(container, value){
            container.filters.push({field: 'land.dimension.area_sf',operator: 'greater_or_equal_to',value: parseInt(value)});
        }
        compile_land_max(container, value){
            container.filters.push({field: 'land.dimension.area_sf',operator: 'less_or_equal_to',value: parseInt(value)});
        }
        
        compile_available_min(container, value){
            container.filters.push({field: 'available_area_sf',operator: 'greater_or_equal_to',value: parseInt(value)});
        }
        compile_available_max(container, value){
            container.filters.push({field: 'available_area_sf',operator: 'less_or_equal_to',value: parseInt(value)});
        }

        compile_categories(container, value){
            container.filters.push({field: 'category_code',operator: 'in',value: value});
        }
        compile_subcategories(container, value){
            container.filters.push({field: 'subcategory_code',operator: 'in',value: value});
        }
        compile_building_categories(container, value){
            container.filters.push({field: 'building.category_code',operator: 'in',value: value});
        }


        getLocationFilterGroup(container){
            let locationGroup = container.filter_groups.find(g => g.name=='location');
            if(locationGroup == null){
                locationGroup = {name:'location', operator:'OR', filters: []};
                container.filter_groups.push(locationGroup);
            }
            return locationGroup;
        }
        compile_location_countries(container, value){
            this.getLocationFilterGroup(container).filters.push({field: 'location.country_code',operator: 'in',value: value});
        }
        compile_location_states(container, value){
            this.getLocationFilterGroup(container).filters.push({field: 'location.state_code',operator: 'in',value: value});
        }
        compile_location_regions(container, value){
            this.getLocationFilterGroup(container).filters.push({field: 'location.region_code',operator: 'in',value: value});
        }
        compile_location_cities(container, value){
            this.getLocationFilterGroup(container).filters.push({field: 'location.city_code',operator: 'in',value: value});
        }
        
        compile_transaction_type(container, value){

            if (value=='sale'){ 
                container.filters.push({field: 'for_sale_flag', operator: 'equal', value: true}); 
            }

            if (value=='rent'){ 
                container.filters.push({field: 'for_rent_flag', operator: 'equal', value: true}); 
            }
            
        }

        compile_parkings(container, value){
            container.filters.push({field:'attributes.PARKING', operator:'greater_than', value: parseInt(value)});
        }
        compile_contract(container, value){
            if($siSearchContext.templates.listing_ages == null) return null;
            const filter = $siSearchContext.templates.listing_ages.find(function($age){ return $age.key == value}).filter;
            if(filter != null ) container.filters.push(filter)
        }
        compile_attributes(container, value){
            if($siSearchContext.templates.listing_attributes == null) return null;

            const filters = Object.keys($siSearchContext.templates.listing_attributes)
                .filter($k => {
                    const lItem = $siSearchContext.templates.listing_attributes[$k];
                    return value.includes(lItem.field);
                })
                .map( ($k) => {
                    const lItem = $siSearchContext.templates.listing_attributes[$k];
                    const lValue = true;

                    return {field: lItem.field, operator : 'equal', value: lValue}
                });

            container.filters.push(...filters);
        }
        compile_states(container, value){
            if($siSearchContext.templates.listing_flags == null) return null;

            const filters = $siSearchContext.templates.listing_flags
                .filter($flag => {
                    return value.includes($flag.key) 
                })
                .map(function($flag){
                    return $flag.filter;
                });      

            container.filters.push(...filters);
        }
        compile_market_type(container, value){
            container.filters.push({field: 'market_codes', operator: 'array_contains', value: value})
        }
    }

    class BrokerFilterBuilder extends FilterBuilder{

        compile_letters(container, value){
            container.filters.push({ field: 'last_name',operator : 'starts_with', value: value });
        }
        compile_licenses(container, value){
            container.filters.push({ field: 'license_type_code', operator: 'in', value: [value] });
        }
        compile_offices(container, value){
            container.filters.push({ field: 'office_ref_number', operator: 'in', value: value });
        }
        compile_agencies(container, value){
            container.filters.push({ field: 'agency.ref_number', operator: 'in', value: value });
        }
        compile_languages(container, value){
            container.filters.push({ field: 'languages', operator: 'array_contains', value: value });
        }

        getLocationFilterGroup(container){
            let locationGroup = container.filter_groups.find(g => g.name=='location');
            if(locationGroup == null){
                locationGroup = {name:'location', operator:'OR', filters: []};
                container.filter_groups.push(locationGroup);
            }
            return locationGroup;
        }

        compile_location_countries(container, value){
             this.getLocationFilterGroup(container).filters.push({field: 'serviced_country_codes',operator: 'array_contains_any_in ',value: value});
        }
        compile_location_states(container, value){
            this.getLocationFilterGroup(container).filters.push({field: 'serviced_state_codes',operator: 'array_contains_any_in ',value: value});
        }
        compile_location_regions(container, value){
            this.getLocationFilterGroup(container).filters.push({field: 'serviced_region_codes',operator: 'array_contains_any_in ',value: value});
        }
        compile_location_cities(container, value){
            this.getLocationFilterGroup(container).filters.push({field: 'serviced_city_codes',operator: 'array_contains_any_in ',value: value});
        }

    }

    class OfficeFilterBuilder extends FilterBuilder{
        compile_agencies(container, value){
            container.filters.push({field: 'agency.ref_number', operator: 'in', value: value});
        }
    }

    class FilterManagerFactory {
        constructor(){
            this.instances = {};
            
        }

        with(alias, configs=null){
            if(alias == undefined) return new FilterFactory();

            if(this.instances[alias] == undefined){
                this.instances[alias] = new FilterFactory(alias);
                this.instances[alias].init(configs);
            }
            
            return this.instances[alias];
        }


    }

    return new FilterManagerFactory();


}]);